import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.Timeout;

import java.lang.reflect.Field;
import java.util.*;

import static org.hamcrest.CoreMatchers.is;

public class Aufgabe1Test {

    @Rule
    public Timeout globalTimeout = Timeout.millis(1000);

    @Test
    public void testPositionEquals() throws Exception {
        Position p = new Position(1,2);
        for (int x = 0; x < 5; x++) {
            for (int y = 0; y < 5; y++) {
                Assert.assertTrue((x == 1 && y == 2) == new Position(x,y).equals(p));
            }
        }
    }

    @Test
    public void testPositionToString() throws Exception {
        Position p = new Position(1,2);
        Assert.assertThat(p.toString().replaceAll("\\s",""), is("(1,2)"));
    }

    @Test
    public void testImageAdd() throws Exception {
        Image image = new Image(3,3);

        image.add(1,1,'#');
        image.add(2,0,'I');
        image.add(new Position(2,2),'#');
        image.add(new Position(0,1),'*');
        image.add(new Position(2,1),'+');

        Field field = Image.class.getDeclaredField("map");
        field.setAccessible(true);

        Map<Position, Character> mapA =  (Map<Position, Character>) field.get(image);
        Map<Position, Character> mapB = new HashMap<Position, Character>();

        mapB.put(new Position(1,1),'#');
        mapB.put(new Position(2,0),'I');
        mapB.put(new Position(2,2),'#');
        mapB.put(new Position(0,1),'*');
        mapB.put(new Position(2,1),'+');

        Assert.assertTrue(mapA.entrySet().containsAll(mapB.entrySet()));
        Assert.assertTrue(mapB.entrySet().containsAll(mapA.entrySet()));
    }

    @Test
    public void testImageSelectRange() throws Exception {
        Image image = new Image(3,3);

        image.add(new Position(1,1),'#');
        image.add(new Position(2,0),'I');
        image.add(new Position(2,2),'#');
        image.add(new Position(0,1),'*');
        image.add(new Position(1,0),'o');
        image.add(new Position(1,2),'W');
        image.add(new Position(2,1),'+');

        List<Position> listA = image.selectRange(2,3);
        List<Position> listB = new LinkedList<>();
        listB.add(new Position(1,1));
        listB.add(new Position(0,1));
        listB.add(new Position(1,0));
        listB.add(new Position(1,2));

        Assert.assertTrue(listA.containsAll(listB) && listB.containsAll(listA));

    }

    @Test
    public void testImageCopyPositions() throws Exception {
        Image image = new Image(3,3);

        image.add(new Position(1,1),'#');
        image.add(new Position(2,0),'I');
        image.add(new Position(2,2),'#');
        image.add(new Position(0,1),'*');
        image.add(new Position(1,0),'o');
        image.add(new Position(1,2),'W');
        image.add(new Position(2,1),'+');

        Image copy = image.copy(2,3);

        Field field = Image.class.getDeclaredField("map");
        field.setAccessible(true);

        Map<Position, Character> mapA =  (Map<Position, Character>) field.get(copy);
        Map<Position, Character> mapB = new HashMap<Position, Character>();

        mapB.put(new Position(1,1),'#');
        mapB.put(new Position(0,1),'*');
        mapB.put(new Position(1,0),'o');
        mapB.put(new Position(1,2),'W');

        Assert.assertTrue(mapA.entrySet().containsAll(mapB.entrySet()) && mapB.entrySet().containsAll(mapA.entrySet()));

    }

    @Test
    public void testImageCopySize() throws Exception {
        Image image = new Image(3,3);

        Image copy = image.copy(2,3);
        Field field = Image.class.getDeclaredField("width");
        field.setAccessible(true);
        int copyWidth =  (int) field.get(copy);
        field = Image.class.getDeclaredField("height");
        field.setAccessible(true);
        int copyHeight =  (int) field.get(copy);

        Assert.assertThat(copyWidth,is(2));
        Assert.assertThat(copyHeight,is(3));

    }

    @Test
    public void testImageToString() throws Exception {
        Image image = new Image(4,3);

        image.add(new Position(1,1),'#');
        image.add(new Position(2,0),'I');
        image.add(new Position(2,2),'#');
        image.add(new Position(0,1),'*');
        image.add(new Position(1,0),'o');
        image.add(new Position(1,2),'W');
        image.add(new Position(2,1),'+');

        String result = ".oI.\n*#+.\n.W#.";

        Assert.assertThat(image.toString().trim(), is(result));

    }

}